"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const tasks_1 = require("@angular-devkit/schematics/tasks");
const utils_1 = require("../utils");
const defaults_1 = require("./defaults");
exports.default = (options) => {
    const addStandardVersionConfigToPackageJson = (options.issuePrefix && options.issuePrefix !== '') || options.standardVersionConfig;
    return schematics_1.chain([
        addDependencies(options),
        options.packages.includes('commitlint') ? addCommitlintConfigFile(options) : schematics_1.noop(),
        options.packages.includes('husky') ? addHuskyConfig(options) : schematics_1.noop(),
        options.packages.includes('commitizen') ? addCommitizenConfig(options) : schematics_1.noop(),
        options.packages.includes('standard-version') ? addNpmRunScript(options) : schematics_1.noop(),
        options.packages.includes('standard-version') && addStandardVersionConfigToPackageJson
            ? standardVersionConfig(options)
            : schematics_1.noop(),
        options.skipInstall ? schematics_1.noop() : installDependencies,
    ]);
};
const addDependencies = (options) => (tree, context) => {
    context.logger.info('Added npm packages as dev dependencies');
    const packageJson = utils_1.getPackageJson(tree);
    let devDepsToAdd = {};
    const map = new Map([
        ['commitlint', defaults_1.DEV_DEPS_TO_ADD],
        ['commitizen', defaults_1.DEV_DEPS_TO_ADD],
        ['husky', defaults_1.DEV_DEPS_TO_ADD],
        ['standard-version', defaults_1.DEV_DEPS_TO_ADD],
    ]);
    for (const [key, value] of map.entries()) {
        const searchKey = key;
        if (options.packages.includes(searchKey)) {
            devDepsToAdd = Object.assign(Object.assign({}, devDepsToAdd), value[searchKey]);
        }
        else {
            context.logger.info(`- Skips adding ${searchKey}`);
        }
    }
    packageJson.devDependencies = Object.assign(Object.assign({}, packageJson.devDependencies), devDepsToAdd);
    utils_1.overwritePackageJson(tree, packageJson);
};
const addNpmRunScript = (options) => (tree, context) => {
    context.logger.info('Added npm script for release');
    const packageJson = utils_1.getPackageJson(tree);
    if (!packageJson.repository && !packageJson.bugs) {
        context.logger.warn(`Neither 'repository' or 'bugs' field found in your 'package.json'. To referencs your releases and issues correctly, you should add the fiels.
Check out: https://github.com/d-koppenhagen/ngx-semantic-version/wiki/FAQ for further information`);
    }
    const configBefore = Object.assign({}, packageJson.scripts);
    const configNew = { release: 'standard-version' };
    packageJson.scripts = utils_1.getMergedPackageJsonConfig(configBefore, configNew, options.overrideConfigurations);
    utils_1.overwritePackageJson(tree, packageJson);
};
const addHuskyConfig = (options) => (tree, context) => {
    addConfig(tree, context, options, 'husky', 'husky', defaults_1.HUSKY_DEFAULTS);
};
const addCommitizenConfig = (options) => (tree, context) => {
    addConfig(tree, context, options, 'commitizen', 'config', defaults_1.COMMITIZEN_DEFAULTS);
};
const standardVersionConfig = (options) => (tree, context) => {
    let defaults = {};
    if (options.standardVersionConfig)
        defaults = Object.assign({}, defaults_1.STANDARD_VERSION_DEFAULTS);
    defaults = Object.assign(Object.assign({}, defaults), { issuePrefixes: [options.issuePrefix || '#'] });
    addConfig(tree, context, options, 'standard-version', 'standard-version', defaults);
};
function addConfig(tree, context, options, what, getPackageJsonPart, defaults) {
    context.logger.info('Added ' + what + ' configuration');
    const packageJson = utils_1.getPackageJson(tree);
    const configBefore = Object.assign({}, packageJson[getPackageJsonPart]);
    packageJson[getPackageJsonPart] = utils_1.getMergedPackageJsonConfig(configBefore, defaults, options.overrideConfigurations);
    utils_1.overwritePackageJson(tree, packageJson);
}
const addCommitlintConfigFile = (options) => (tree, context) => {
    context.logger.info('Added commitlint configuration file');
    const sourceTemplates = schematics_1.url('./files');
    const sourceParameterizedTemplates = schematics_1.apply(sourceTemplates, [
        schematics_1.template({
            issuePrefix: options.issuePrefix || '',
        }),
        schematics_1.forEach((fileEntry) => {
            // override existing files if force flag has been set
            if (options.overrideConfigurations && tree.exists(fileEntry.path)) {
                tree.overwrite(fileEntry.path, fileEntry.content);
                return null;
            }
            return fileEntry;
        }),
    ]);
    return schematics_1.mergeWith(sourceParameterizedTemplates)(tree, context);
};
const installDependencies = () => (_tree, context) => {
    context.logger.info('Installs npm dependencies');
    context.addTask(new tasks_1.NodePackageInstallTask());
};
//# sourceMappingURL=index.js.map